@@ -9,6 +9,8 @@ This code was based on the work of others found on the internet:
 3. https://groups.google.com/forum/?fromgroups=#!topic/django-users/cjzloTUwmS0
 4. https://groups.google.com/forum/?fromgroups=#!topic/django-users/vVprMpsAnPo
 '''
+from __future__ import unicode_literals
+import six
 from django.db.models.aggregates import Aggregate as DjangoAggregate
 from django.db.models.sql.aggregates import Aggregate as DjangoSqlAggregate
 
@@ -23,7 +25,10 @@ class SqlAggregate(DjangoSqlAggregate):
     def relabel_aliases(self, change_map):
         super(SqlAggregate, self).relabel_aliases(change_map)
         if self.has_condition:
-            self.condition.relabel_aliases(change_map)
+            condition_change_map = dict((k, v) for k, v in \
+                change_map.iteritems() if k in self.condition.query.alias_map
+            )
+            self.condition.query.change_aliases(condition_change_map)
 
     def as_sql(self, qn, connection):
         if self.has_condition:
@@ -44,7 +49,7 @@ class SqlAggregate(DjangoSqlAggregate):
         def escape(value):
             if isinstance(value, bool):
                 value = str(int(value))
-            if isinstance(value, basestring):
+            if isinstance(value, six.string_types):
                 # Escape params used with LIKE
                 if '%' in value:
                     value = value.replace('%', '%%')
@@ -94,9 +99,32 @@ class Aggregate(DjangoAggregate):
         self.only = only
         self.condition = None
 
+    def _get_fields_from_Q(self, q):
+        fields = []
+        for child in q.children:
+            if hasattr(child, 'children'):
+                fields.extend(self._get_fields_from_Q(child))
+            else:
+                fields.append(child)
+        return fields
+
     def add_to_query(self, query, alias, col, source, is_summary):
         if self.only:
             self.condition = query.model._default_manager.filter(self.only)
+            for child in self._get_fields_from_Q(self.only):
+                field_list = child[0].split('__')
+                # Pop off the last field if it's a query term ('gte', 'contains', 'isnull', etc.)
+                if field_list[-1] in query.query_terms:
+                    field_list.pop()
+                # setup_joins have different returns in Django 1.5 and 1.6, but the order of what we need remains.
+                result = query.setup_joins(field_list, query.model._meta, query.get_initial_alias(), None)
+                join_list = result[3]
+                # Django 1.5+
+                if hasattr(query, 'promote_joins'):
+                    query.promote_joins(join_list, True)
+                # Django < 1.5
+                elif hasattr(query, 'promote_alias_chain'):
+                    query.promote_alias_chain(join_list, True)
 
         aggregate = self.sql_klass(col, source=source, is_summary=is_summary, condition=self.condition, **self.extra)
         query.aggregates[alias] = aggregate
